#ifndef __GALLEGRO__
#define __GALLEGRO__

/* Tipos de dados */
#include "cores.h"
#include "pontos.h"
#include "texto.h"

/* Allegro */
#include <allegro5/allegro.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_native_dialog.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_audio.h> 
#include <allegro5/allegro_acodec.h> 

/* Macros de Pixels para uso no Allegro */
#define COLORS 256
#define PIXEL(r,g,b) (al_map_rgb((unsigned char) r, (unsigned char) g, (unsigned char) b))
#define BPIXEL(pixel) PIXEL(pixel.R, pixel.G, pixel.B)

enum TECLAS_DIR { KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT };

typedef enum { SQUARE, CIRCLE, TRIANG, PENTAG } Shape;

typedef struct {
  ALLEGRO_EVENT_QUEUE *event_queue;
  ALLEGRO_TIMER *timer;
  ALLEGRO_DISPLAY *display;
  ALLEGRO_SAMPLE *som;
  ALLEGRO_DISPLAY_MODE disp_data;
  int dot_width, dot_height, // dimensão máxima de uma unidade visual
    width_dots, height_dots; // quantidade de unidades visuais
} Window;

typedef struct {
  ALLEGRO_BITMAP *bmp;
  int x, y, w, h;
} BITMAP;
  
typedef struct {
  BITMAP bloco;
  char *txt;
  ALLEGRO_FONT *font;
} TEXTMAP;

  
/* Inicia sistema grafico Allegro 5.0. 'width' e 'height' são as dimensões 
   reais   da  área   de   trabalho,  em   termos   de  unidades  visuais
   (dots).
*/
Window graphinit(int width, int height);

/* Finaliza sistema grafico Allegro 5.0 */
void graphdeinit(Window win);

/* Cria  região em  display para  exibir imagem PNG  ou JPG  em ambiente
   ALLEGRO a partir de arquivo
 */
ALLEGRO_BITMAP *imagemArq(char *nomeArqImg, int width, int height,Window win);

/* Cria  um cursor (bloco retangular) em ambiente ALLEGRO. */
ALLEGRO_BITMAP *blocoBitmap(int width, int height, ALLEGRO_COLOR color, Window win);

/* Cria bitmap para caber 'texto'.
   Bitmap terá dimensão 'width x height' ou igual à dimensão da fonte, 
   dependendo de quem for maior.
   Retorna um TEXMAP contendo o bitmap pronto para ser preenchido com 'texto'.
*/
TEXTMAP *blocoTexto(char *texto, ALLEGRO_FONT *fonte, ALLEGRO_COLOR cor_bloco, ALLEGRO_COLOR cor_texto, int width, int height, Window win);

/* Cria um bitmap com 'texto', centralizado no bitmap.
   Usa função 'blocoText()' para criar bitmap e depois
   desenha 'texto' dentro.
   Retorna TEXTMAP contendo o bitmap com o texto dentro.
*/
TEXTMAP *centerText(char *texto, ALLEGRO_FONT *fonte, ALLEGRO_COLOR cor_bloco, ALLEGRO_COLOR cor_texto, int width, int height, Window win);

/* Cria um bitmap com 'texto', alinhado à esquerda no bitmap.
   Usa função 'blocoText()' para criar bitmap e depois
   desenha 'texto' dentro.
   Retorna TEXTMAP contendo o bitmap com o texto dentro.
*/
TEXTMAP *leftText(char *texto, ALLEGRO_FONT *fonte, ALLEGRO_COLOR cor_bloco, ALLEGRO_COLOR cor_texto, int width, int height, Window win);

/* Cria região em display para exibir imagem em ambiente ALLEGRO a partir 
   uma imagem PPM. Esta consiste de um vetor linear de valores do tipo 
   'Pixel', representando uma matriz width x height.
   Se arg. 'imgPPM' é NULL, apaga imagem atual e BITMAP fica na cor NEVE.
 */
ALLEGRO_BITMAP *imagemPPM(Pixel *imgPPM, int width, int height, Window win);

/* Muda a imagem em um BITMAP para imagem em formato PPM
   Se arg. 'imgPPM' éNULL, apaga imagem atual e BITMAP fica na cor NEVE.
 */
void mudaImgPPM(BITMAP imagem, Pixel *imgPPM, int width, int height, Window win);

/* Retorna 1 se as coordenadas x,y estão dentro da imagem, 0 caso contrário. */
bool is_inside_bitmap (BITMAP img, int x, int y);

/* Altera o PERÍODO do timer de Window, multiplicando o período 
   atual pelo fator.
   Nota: Ao criar Window, o período inicial é definido como sendo 
         '1.0 / win.disp_data.refresh_rate'
*/
void setTimerSpeed(Window win, double fator);

/* Lista de cores 
   Fonte: http://pt.wikipedia.org/wiki/Anexo:Lista_de_cores
*/
#define ACAFRAO                   BPIXEL((*pickColor("ACAFRAO")))
#define AGUA_MARINHA              BPIXEL((*pickColor("AGUA_MARINHA")))
#define AGUA_MARINHA_MEDIA        BPIXEL((*pickColor("AGUA_MARINHA_MEDIA")))
#define ALIZARINA                 BPIXEL((*pickColor("ALIZARINA")))
#define AMARELO                   BPIXEL((*pickColor("AMARELO")))
#define AMARELO_CLARO             BPIXEL((*pickColor("AMARELO_CLARO")))
#define AMARELO_ESVERDEADO        BPIXEL((*pickColor("AMARELO_ESVERDEADO")))
#define AMARELO_OURO_CLARO        BPIXEL((*pickColor("AMARELO_OURO_CLARO")))
#define AMARELO_QUEIMADO          BPIXEL((*pickColor("AMARELO_QUEIMADO")))
#define AMBAR                     BPIXEL((*pickColor("AMBAR")))
#define AMEIXA                    BPIXEL((*pickColor("AMEIXA")))
#define AMENDOA                   BPIXEL((*pickColor("AMENDOA")))
#define AMETISTA                  BPIXEL((*pickColor("AMETISTA")))
#define ASPARGO                   BPIXEL((*pickColor("ASPARGO")))
#define AZUL                      BPIXEL((*pickColor("AZUL")))
#define AZUL_ACO                  BPIXEL((*pickColor("AZUL_ACO")))
#define AZUL_ACO_CLARO            BPIXEL((*pickColor("AZUL_ACO_CLARO")))
#define AZUL_ALICE                BPIXEL((*pickColor("AZUL_ALICE")))
#define AZUL_ARDOSIA              BPIXEL((*pickColor("AZUL_ARDOSIA")))
#define AZUL_ARDOSIA_CLARO        BPIXEL((*pickColor("AZUL_ARDOSIA_CLARO")))
#define AZUL_ARDOSIA_ESCURO       BPIXEL((*pickColor("AZUL_ARDOSIA_ESCURO")))
#define AZUL_ARDOSIA_MEDIO        BPIXEL((*pickColor("AZUL_ARDOSIA_MEDIO")))
#define AZUL_AREADO               BPIXEL((*pickColor("AZUL_AREADO")))
#define AZUL_CADETE               BPIXEL((*pickColor("AZUL_CADETE")))
#define AZUL_CAMARADA             BPIXEL((*pickColor("AZUL_CAMARADA")))
#define AZUL_CELESTE              BPIXEL((*pickColor("AZUL_CELESTE")))
#define AZUL_CELESTE_PALIDO       BPIXEL((*pickColor("AZUL_CELESTE_PALIDO")))
#define AZUL_CELESTE_BRILHANTE    BPIXEL((*pickColor("AZUL_CELESTE_BRILHANTE")))
#define AZUL_CELESTE_CLARO        BPIXEL((*pickColor("AZUL_CELESTE_CLARO")))
#define AZUL_CELESTE_PROFUNDO     BPIXEL((*pickColor("AZUL_CELESTE_PROFUNDO")))
#define AZUL_CLARO                BPIXEL((*pickColor("AZUL_CLARO")))
#define AZUL_COBALTO              BPIXEL((*pickColor("AZUL_COBALTO")))
#define AZUL_ESCURO               BPIXEL((*pickColor("AZUL_ESCURO")))
#define AZUL_FLOR_DE_MILHO        BPIXEL((*pickColor("AZUL_FLOR_DE_MILHO")))
#define AZUL_FORCA_AEREA          BPIXEL((*pickColor("AZUL_FORCA_AEREA")))
#define AZUL_FURTIVO              BPIXEL((*pickColor("AZUL_FURTIVO")))
#define AZUL_MARINHO              BPIXEL((*pickColor("AZUL_MARINHO")))
#define AZUL_MEDIO                BPIXEL((*pickColor("AZUL_MEDIO")))
#define AZUL_MEIA_NOITE           BPIXEL((*pickColor("AZUL_MEIA_NOITE")))
#define AZUL_NAVAL                BPIXEL((*pickColor("AZUL_NAVAL")))
#define AZUL_PETROLEO             BPIXEL((*pickColor("AZUL_PETROLEO")))
#define AZUL_REAL                 BPIXEL((*pickColor("AZUL_REAL")))
#define AZUL_TALCO                BPIXEL((*pickColor("AZUL_TALCO")))
#define AZUL_TAPARUERE            BPIXEL((*pickColor("AZUL_TAPARUERE")))
#define AZUL_VIOLETA              BPIXEL((*pickColor("AZUL_VIOLETA")))
#define BEGE                      BPIXEL((*pickColor("BEGE")))
#define BORDO                     BPIXEL((*pickColor("BORDO")))
#define BORGONHA                  BPIXEL((*pickColor("BORGONHA")))
#define BRANCO                    BPIXEL((*pickColor("BRANCO")))
#define BRANCO_ANTIGO             BPIXEL((*pickColor("BRANCO_ANTIGO")))
#define BRANCO_FANTASMA           BPIXEL((*pickColor("BRANCO_FANTASMA")))
#define BRANCO_FLORAL             BPIXEL((*pickColor("BRANCO_FLORAL")))
#define BRANCO_FUMACA             BPIXEL((*pickColor("BRANCO_FUMACA")))
#define BRANCO_NAVAJO             BPIXEL((*pickColor("BRANCO_NAVAJO")))
#define BRONZE                    BPIXEL((*pickColor("BRONZE")))
#define CAQUI                     BPIXEL((*pickColor("CAQUI")))
#define CAQUI_ESCURO              BPIXEL((*pickColor("CAQUI_ESCURO")))
#define CARAMELO                  BPIXEL((*pickColor("CARAMELO")))
#define CARDO                     BPIXEL((*pickColor("CARDO")))
#define CARMESIM                  BPIXEL((*pickColor("CARMESIM")))
#define CARMIM                    BPIXEL((*pickColor("CARMIM")))
#define CARMIM_CARNACEO           BPIXEL((*pickColor("CARMIM_CARNACEO")))
#define CASTANHO_AVERMELHADO      BPIXEL((*pickColor("CASTANHO_AVERMELHADO")))
#define CASTANHO_CLARO            BPIXEL((*pickColor("CASTANHO_CLARO")))
#define CENOURA                   BPIXEL((*pickColor("CENOURA")))
#define CEREJA                    BPIXEL((*pickColor("CEREJA")))
#define CEREJA_HOLLYWOOD          BPIXEL((*pickColor("CEREJA_HOLLYWOOD")))
#define CHOCOLATE                 BPIXEL((*pickColor("CHOCOLATE")))
#define CIANO                     BPIXEL((*pickColor("CIANO")))
#define CIANO_CLARO               BPIXEL((*pickColor("CIANO_CLARO")))
#define CIANO_ESCURO              BPIXEL((*pickColor("CIANO_ESCURO")))
#define CINZA                     BPIXEL((*pickColor("CINZA")))
#define CINZA_ARDOSIA             BPIXEL((*pickColor("CINZA_ARDOSIA")))
#define CINZA_ARDOSIA_CLARO       BPIXEL((*pickColor("CINZA_ARDOSIA_CLARO")))
#define CINZA_ARDOSIA_ESCURO      BPIXEL((*pickColor("CINZA_ARDOSIA_ESCURO")))
#define CINZA_CLARO               BPIXEL((*pickColor("CINZA_CLARO")))
#define CINZA_ESCURO              BPIXEL((*pickColor("CINZA_ESCURO")))
#define CINZA_FOSCO               BPIXEL((*pickColor("CINZA_FOSCO")))
#define CINZA_MEDIO               BPIXEL((*pickColor("CINZA_MEDIO")))
#define COBRE                     BPIXEL((*pickColor("COBRE")))
#define CONCHA                    BPIXEL((*pickColor("CONCHA")))
#define CORAL                     BPIXEL((*pickColor("CORAL")))
#define CORAL_CLARO               BPIXEL((*pickColor("CORAL_CLARO")))
#define COURO                     BPIXEL((*pickColor("COURO")))
#define CREME                     BPIXEL((*pickColor("CREME")))
#define CREME_DE_MARISCO          BPIXEL((*pickColor("CREME_DE_MARISCO")))
#define CREME_DE_MENTA            BPIXEL((*pickColor("CREME_DE_MENTA")))
#define DOURADO                   BPIXEL((*pickColor("DOURADO")))
#define DOURADO_ESCURO            BPIXEL((*pickColor("DOURADO_ESCURO")))
#define DOURADA_LINHACA           BPIXEL((*pickColor("DOURADA_LINHACA")))
#define DOURADO_PALIDO            BPIXEL((*pickColor("DOURADO_PALIDO")))
#define ESCARLATE                 BPIXEL((*pickColor("ESCARLATE")))
#define ESMERALDA                 BPIXEL((*pickColor("ESMERALDA")))
#define FELDSPATO                 BPIXEL((*pickColor("FELDSPATO")))
#define FERRUGEM                  BPIXEL((*pickColor("FERRUGEM")))
#define FULIGEM                   BPIXEL((*pickColor("FULIGEM")))
#define GRENA                     BPIXEL((*pickColor("GRENA")))
#define HERBAL                    BPIXEL((*pickColor("HERBAL")))
#define INDIGO                    BPIXEL((*pickColor("INDIGO")))
#define JADE                      BPIXEL((*pickColor("JADE")))
#define JAMBO                     BPIXEL((*pickColor("JAMBO")))
#define LARANJA                   BPIXEL((*pickColor("LARANJA")))
#define LARANJA_ESCURO            BPIXEL((*pickColor("LARANJA_ESCURO")))
#define LAVANDA                   BPIXEL((*pickColor("LAVANDA")))
#define LAVANDA_AVERMELHADA       BPIXEL((*pickColor("LAVANDA_AVERMELHADA")))
#define LILAS                     BPIXEL((*pickColor("LILAS")))
#define LIMA                      BPIXEL((*pickColor("LIMA")))
#define LINHO                     BPIXEL((*pickColor("LINHO")))
#define MADEIRA                   BPIXEL((*pickColor("MADEIRA")))
#define MAGENTA                   BPIXEL((*pickColor("MAGENTA")))
#define MAGENTA_ESCURO            BPIXEL((*pickColor("MAGENTA_ESCURO")))
#define MALVA                     BPIXEL((*pickColor("MALVA")))
#define MAMAO_BATIDO              BPIXEL((*pickColor("MAMAO_BATIDO")))
#define MANA                      BPIXEL((*pickColor("MANA")))
#define MARFIM                    BPIXEL((*pickColor("MARFIM")))
#define MARROM                    BPIXEL((*pickColor("MARROM")))
#define MARROM_AMARELADO          BPIXEL((*pickColor("MARROM_AMARELADO")))
#define MARROM_CLARO              BPIXEL((*pickColor("MARROM_CLARO")))
#define MARROM_ROSADO             BPIXEL((*pickColor("MARROM_ROSADO")))
#define MARROM_SELA               BPIXEL((*pickColor("MARROM_SELA")))
#define MILHO                     BPIXEL((*pickColor("MILHO")))
#define MILHO_CLARO               BPIXEL((*pickColor("MILHO_CLARO")))
#define MOCASSIM                  BPIXEL((*pickColor("MOCASSIM")))
#define MOSTARDA                  BPIXEL((*pickColor("MOSTARDA")))
#define NEVE                      BPIXEL((*pickColor("NEVE")))
#define OCRE                      BPIXEL((*pickColor("OCRE")))
#define ORQUIDEA                  BPIXEL((*pickColor("ORQUIDEA")))
#define ORQUIDEA_ESCURA           BPIXEL((*pickColor("ORQUIDEA_ESCURA")))
#define ORQUIDEA_MEDIA            BPIXEL((*pickColor("ORQUIDEA_MEDIA")))
#define OURO                      BPIXEL((*pickColor("OURO")))
#define PESSEGO                   BPIXEL((*pickColor("PESSEGO")))
#define PRATA                     BPIXEL((*pickColor("PRATA")))
#define PRETO                     BPIXEL((*pickColor("PRETO")))
#define PURPURA                   BPIXEL((*pickColor("PURPURA")))
#define PURPURA_MEDIA             BPIXEL((*pickColor("PURPURA_MEDIA")))
#define QUANTUM                   BPIXEL((*pickColor("QUANTUM")))
#define RENDA_ANTIGA              BPIXEL((*pickColor("RENDA_ANTIGA")))
#define ROSA                      BPIXEL((*pickColor("ROSA")))
#define ROSA_BRILHANTE            BPIXEL((*pickColor("ROSA_BRILHANTE")))
#define ROSA_CHOQUE               BPIXEL((*pickColor("ROSA_CHOQUE")))
#define ROSA_CLARO                BPIXEL((*pickColor("ROSA_CLARO")))
#define ROSA_EMBACADO             BPIXEL((*pickColor("ROSA_EMBACADO")))
#define ROSA_FORTE                BPIXEL((*pickColor("ROSA_FORTE")))
#define ROSA_PROFUNDO             BPIXEL((*pickColor("ROSA_PROFUNDO")))
#define ROXO                      BPIXEL((*pickColor("ROXO")))
#define RUTILO                    BPIXEL((*pickColor("RUTILO")))
#define SALMAO                    BPIXEL((*pickColor("SALMAO")))
#define SALMAO_CLARO              BPIXEL((*pickColor("SALMAO_CLARO")))
#define SALMAO_ESCURO             BPIXEL((*pickColor("SALMAO_ESCURO")))
#define SEPIA                     BPIXEL((*pickColor("SEPIA")))
#define SIENA                     BPIXEL((*pickColor("SIENA")))
#define TERRACOTA                 BPIXEL((*pickColor("TERRACOTA")))
#define TIJOLO_REFRATARIO         BPIXEL((*pickColor("TIJOLO_REFRATARIO")))
#define TOMATE                    BPIXEL((*pickColor("TOMATE")))
#define TRIASSICO                 BPIXEL((*pickColor("TRIASSICO")))
#define TRIGO                     BPIXEL((*pickColor("TRIGO")))
#define TURQUESA                  BPIXEL((*pickColor("TURQUESA")))
#define TURQUESA_ESCURA           BPIXEL((*pickColor("TURQUESA_ESCURA")))
#define TURQUESA_MEDIA            BPIXEL((*pickColor("TURQUESA_MEDIA")))
#define TURQUESA_PALIDA           BPIXEL((*pickColor("TURQUESA_PALIDA")))
#define URUCUM                    BPIXEL((*pickColor("URUCUM")))
#define VERDE                     BPIXEL((*pickColor("VERDE")))
#define VERDE_AMARELADO           BPIXEL((*pickColor("VERDE_AMARELADO")))
#define VERDE_AZULADO             BPIXEL((*pickColor("VERDE_AZULADO")))
#define VERDE_CLARO               BPIXEL((*pickColor("VERDE_CLARO")))
#define VERDE_ESCURO              BPIXEL((*pickColor("VERDE_ESCURO")))
#define VERDE_FLORESTA            BPIXEL((*pickColor("VERDE_FLORESTA")))
#define VERDE_FLUORESCENTE        BPIXEL((*pickColor("VERDE_FLUORESCENTE")))
#define VERDE_GRAMA               BPIXEL((*pickColor("VERDE_GRAMA")))
#define VERDE_LIMA                BPIXEL((*pickColor("VERDE_LIMA")))
#define VERDE_LIMAO               BPIXEL((*pickColor("VERDE_LIMAO")))
#define VERDE_MAR_CLARO           BPIXEL((*pickColor("VERDE_MAR_CLARO")))
#define VERDE_MAR_ESCURO          BPIXEL((*pickColor("VERDE_MAR_ESCURO")))
#define VERDE_MAR_MEDIO           BPIXEL((*pickColor("VERDE_MAR_MEDIO")))
#define VERDE_MILITAR             BPIXEL((*pickColor("VERDE_MILITAR")))
#define VERDE_OLIVA               BPIXEL((*pickColor("VERDE_OLIVA")))
#define VERDE_OLIVA_ESCURA        BPIXEL((*pickColor("VERDE_OLIVA_ESCURA")))
#define VERDE_OLIVA_PARDO         BPIXEL((*pickColor("VERDE_OLIVA_PARDO")))
#define VERDE_PALIDO              BPIXEL((*pickColor("VERDE_PALIDO")))
#define VERDE_PARIS               BPIXEL((*pickColor("VERDE_PARIS")))
#define VERDE_PRIMAVERA           BPIXEL((*pickColor("VERDE_PRIMAVERA")))
#define VERDE_PRIMAVERA_MEDIO     BPIXEL((*pickColor("VERDE_PRIMAVERA_MEDIO")))
#define VERMELHO                  BPIXEL((*pickColor("VERMELHO")))
#define VERMELHO_ESCURO           BPIXEL((*pickColor("VERMELHO_ESCURO")))
#define VERMELHO_INDIANO          BPIXEL((*pickColor("VERMELHO_INDIANO")))
#define VERMELHO_VIOLETA          BPIXEL((*pickColor("VERMELHO_VIOLETA")))
#define VERMELHO_VIOLETA_MEDIO    BPIXEL((*pickColor("VERMELHO_VIOLETA_MEDIO")))
#define VERMELHO_VIOLETA_PALIDO   BPIXEL((*pickColor("VERMELHO_VIOLETA_PALIDO")))
#define VIOLETA                   BPIXEL((*pickColor("VIOLETA")))
#define VIOLETA_CLARO             BPIXEL((*pickColor("VIOLETA_CLARO")))
#define VIOLETA_ESCURO            BPIXEL((*pickColor("VIOLETA_ESCURO")))

#endif // __GALLEGRO__
